/*********************************************************
 * ml_help.c
 *
 * Copyright (c) Christian Ensel
 *
 * Help for pv-merlin-modes and strokes 
 *********************************************************/

#include <stdrom.h>
#include "define.h"
#include "libc.h"
#include "md.h"

#include "mod_symb.h"
#include "ml_help.h"
#include "pattern.h"

extern Pattern FAR *models[];

void merlinhelpmenu()
{
    int result;
    byte cmnt[]="General Help\xfe"
                "Writing Modes\xfe"
                "Letters\x0fe"
                "Numeric Mode\x0fe"
                "Extended Characters\x0fe"
                "Extended Numeric Symbols\x0fe"
                "Accents and Umlauts\x0fe"
                "About\xff";
    byte rtv[10+1] = { HELPMODE_GENERAL, HELPMODE_MODES, HELPMODE_ALPHA, HELPMODE_NUM, 
                       HELPMODE_EXT, HELPMODE_EXT_NUM, HELPMODE_ACC, HELPMODE_ABOUT, 0xff};
    SLW2_ST slw;
    slw.x=9;
    slw.y=25;
    slw.xs=138;
    slw.ity=9;
    slw.np=0;
    slw.t_xs=3;
    slw.rtv=rtv;
    slw.cmnt=cmnt;

    while ( (result=LibSelWindowExt(&slw)) != 255) /* if anything else but <esc> has been pressed */
        merlinhelp(result);
}               

#define WINX 3
#define WINY 3


/* draw a pattern */
void drawOnePattern(int x, int y, int model, int nr )
{
    int i,downshift=0;
    Pattern FAR *pat = &models[model][nr];
    
    if ( pat->ascii == 0x08 || pat->ascii == 0x20 || pat->ascii == 0x14
         || pat->ascii == '_' || pat->ascii == '-' || pat->ascii == '=' )
         downshift = 11;
    for (i = 0; i < pat->nPoints-1; i++) {
	    LibGdsLine( WINX + x + (pat->pv[i].x >>2)  , WINY + y+ (pat->pv[i].y >>2) + downshift , 
	                WINX + x + (pat->pv[i+1].x >>2), WINY + y+ (pat->pv[i+1].y >>2) + downshift);
	}
    LibBox( x + (pat->pv[0].x>>2)-1 , y + (pat->pv[0].y>>2)-1 + downshift, 3, 3, 1);
    LibDotOn( x + (pat->pv[0].x>>2) , y + (pat->pv[0].y>>2) + downshift);
    
    /*if ( pat->ascii == UP_STROKE )
        LibPutProStr(IB_PFONT2, x, y+19, "CAP", 159 );
    else if ( pat->ascii == NUM_STROKE )
        LibPutProStr(IB_PFONT2, x, y+19, "NUM", 159 );
    else */ if ( pat->ascii == COPY_STROKE )
        LibPutProStr(IB_PFONT1, WINX + x, WINY + y+27, "Copy", 159 );
    else if ( pat->ascii == PASTE_STROKE )
        LibPutProStr(IB_PFONT1, WINX + x, WINY + y+27, "Paste", 159 );
    else if ( pat->ascii == 0x08 )
        LibPutProStr(IB_PFONT1, WINX + x, WINY + y+24,  "<-BS", 159 );
    else if ( pat->ascii == 0x20 )
        LibPutProStr(IB_PFONT1, WINX + x, WINY + y+24,  "Space", 159 );
    else if ( pat->ascii == 0x0D )
        LibPutProStr(IB_PFONT1, WINX + x, WINY + y+24, "Enter", 159 );
    else if ( (pat->ascii >= 0x11 && pat->ascii <= 0x14) || pat->ascii=='"' )
        LibPutProFont(IB_PFONT2, pat->ascii, WINX + x, WINY + y+27 );
    else 
        LibPutProFont(IB_PFONT2, pat->ascii, WINX + x, WINY + y+24 );
}

void drawPatterns(int xs, int ys, int model, int nr1, int nre )
{
    int n,i;
    for ( n=nr1, i=0; n<=nre; n++, i++) {
        drawOnePattern( xs + (i%5)*31, ys + (i/5)*36, model,n);
	}
}


#define OBJ_HELPWIN 0x8FFE
static TCHTBL far helpwin_tchtbl[] = {
       { 0,0, 159, 159, ACT_BREAK_IN, OBJ_HELPWIN, 0x000 },
       { 0,0,0,0,ACT_NONE,OBJ_END,0x0000 }
};

/* returns TRUE for normal touch, FALSE for <esc> */
int waitfornextpage()
{
    TCHSTS tsts;
    int res=TRUE;
    LibPutDisp();
    LibTchStackPush(NULL);
    LibTchStackPush(TchHardIcon);
    LibTchStackPush(helpwin_tchtbl);
    do {
        LibTchWait(&tsts);
        /*LibPutMsgDlg2("%i",tsts.act);*/
    } while ( tsts.act != ACT_BREAK && tsts.act != ACT_BREAK_IN && tsts.obj!=OBJ_HIC_ESC);
    LibTchStackPop();
    LibTchStackPop();
    LibTchStackPop();
    if (tsts.obj==OBJ_HIC_ESC)
        res = FALSE;
    LibClrBox(1,1,152,152);
    LibPutDisp();
    return res;
}


void merlinhelp(t_helpmode m)
{
    LibOpenWindowS( ON, WINX, WINY, 154, 154 );
    switch (m) {
        case HELPMODE_GENERAL:
            LibPutProStr( IB_PFONT2, 5,  5, "General information:", 152 );
            LibPutProStr( IB_PFONT1, 5, 16, "PV-Merlin uses no separate", 152 );
            LibPutProStr( IB_PFONT1, 5, 24, "input field to enter text.", 152 );
            LibPutProStr( IB_PFONT1, 5, 35, "Simply draw characters on the", 152 );
            LibPutProStr( IB_PFONT1, 5, 43, "text area. The character will be", 152 );
            LibPutProStr( IB_PFONT1, 5, 51, "inserted at the cursor position.", 152 );
            LibPutProStr( IB_PFONT1, 5, 61, "Hold the pen down for a short", 152 );
            LibPutProStr( IB_PFONT1, 5, 70, "moment to place the cursor or", 152 );
            LibPutProStr( IB_PFONT1, 5, 78, "to select text.", 152 );
            LibPutProStr( IB_PFONT1, 5, 89, "Please remember that PV-Merlin", 152 );
            LibPutProStr( IB_PFONT1, 5, 97, "does not perform real hand-", 152 );
            LibPutProStr( IB_PFONT1, 5,105, "writing recognition, but your", 152 );
            LibPutProStr( IB_PFONT1, 5,113, "strokes have to follow certain", 152 );
            LibPutProStr( IB_PFONT1, 5,121, "pre-set patterns. However,these", 152 );
            LibPutProStr( IB_PFONT1, 5,129, "patterns look mainly like normal", 152 );
            LibPutProStr( IB_PFONT1, 5,137, "lower-case letters and therefore", 152 );
            LibPutProStr( IB_PFONT1, 5,145, "are easy to write.", 152 );
            if (!waitfornextpage())
                break;

            LibPutProStr( IB_PFONT2, 5,  5, "Hints:", 152 );
            LibPutProStr( IB_PFONT1, 5, 16, "When writing characters, do not", 152 );
            LibPutProStr( IB_PFONT1, 5, 24, "hold the pen down and then", 152 );
            LibPutProStr( IB_PFONT1, 5, 32, "start to draw the strokes,", 152 );
            LibPutProStr( IB_PFONT1, 5, 40, "because if you wait too long,", 152 );
            LibPutProStr( IB_PFONT1, 5, 48, "you will start to select text", 152 );
            LibPutProStr( IB_PFONT1, 5, 56, "instead. Better move the pen", 152 );
            LibPutProStr( IB_PFONT1, 5, 64, "immediately after or almost", 152 );
            LibPutProStr( IB_PFONT1, 5, 72, "while you start to touch the ", 152 );
            LibPutProStr( IB_PFONT1, 5, 80, "display.", 152 );
            LibPutProStr( IB_PFONT1, 5, 91, "Pressing <Esc> will save the", 152 );
            LibPutProStr( IB_PFONT1, 5, 99, "current text and show the PV-", 152 );
            LibPutProStr( IB_PFONT1, 5,107, "main menu. So, if you like what", 152 );
            LibPutProStr( IB_PFONT1, 5,115, "you wrote, do not directly", 152 );
            LibPutProStr( IB_PFONT1, 5,123, "switch to other applications, but", 152 );
            LibPutProStr( IB_PFONT1, 5,131, "press <Esc> first.", 152 );
            waitfornextpage();
            break;

        case HELPMODE_MODES:
            LibPutProStr( IB_PFONT2, 5,  5, "Writing Modes:", 152 );
            LibPutProStr( IB_PFONT1, 5, 16, "In normal writing mode you can", 152 );
            LibPutProStr( IB_PFONT1, 5, 24, "write any lowercase letter and", 152 );
            LibPutProStr( IB_PFONT1, 5, 32, "some special strokes for 'space'", 152 );
            LibPutProStr( IB_PFONT1, 5, 40, "'new line', 'Copy', 'Paste', etc.", 152 );
            LibPutProStr( IB_PFONT1, 5, 51, "There are 4 more writing modes", 152 );
            LibPutProStr( IB_PFONT1, 5, 59, "for 'capital letters', 'numbers'", 152 );
            LibPutProStr( IB_PFONT1, 5, 67, "and two sets of extended", 152 );
            LibPutProStr( IB_PFONT1, 5, 75, "characters.", 152 );
            LibPutProStr( IB_PFONT1, 5, 86, "Modes are changed by writing", 152 );
            LibPutProStr( IB_PFONT1, 5, 94, "certain simple strokes or by", 152 );
            LibPutProStr( IB_PFONT1, 5,102, "clicking one or more times", 152 );
            LibPutProStr( IB_PFONT1, 5,110, "on one of the three mode indi-", 152 );
            LibPutProStr( IB_PFONT1, 5,118, "cators above the scroll-bar.", 152 );
            LibPutProStr( IB_PFONT1,120,145, "more ...", 152 );
            if (!waitfornextpage())
                break;

            LibPutProStr( IB_PFONT2, 5,  5, "Capital letters:", 152 );
            LibPutProStr( IB_PFONT1, 22, 15, "Draw a simple upward-", 152 );
            LibPutProStr( IB_PFONT1, 22, 23, "stroke to write a single", 152 );
            LibPutProStr( IB_PFONT1, 22, 31, "capital letter.", 152 );
            LibPutProStr( IB_PFONT1, 22, 39, "Two upward-strokes lock", 152 );
            LibPutProStr( IB_PFONT1, 22, 47, "the mode. Draw another", 152 );
            LibPutProStr( IB_PFONT1, 22, 55, "upward- stroke to unlock", 152 );
            LibPutProStr( IB_PFONT1, 22, 63, "and switch back.", 152 );
            drawOnePattern( 9, 13, 0, 37 ); /* UP_STROKE */
            drawOnePattern( 6, 39, 0, 37 ); /* UP_STROKE */
            drawOnePattern(14, 39, 0, 37 ); /* UP_STROKE */
            LibPutProStr( IB_PFONT1, 5, 71, "Alternatively, you can click once", 152 );
            LibPutProStr( IB_PFONT1, 5, 79, "or twice on the upper mode-indi-", 152 );
            LibPutProStr( IB_PFONT1, 5, 87, "cator. It shows a bold letter 'a'", 152 );
            LibPutProStr( IB_PFONT1, 5, 95, "for lower-case and 'A' for", 152 );
            LibPutProStr( IB_PFONT1, 5,103, "capital mode.", 152 );

            LibPutProStr( IB_PFONT2, 5,114, "Extended Characters:", 152 );
            LibPutProStr( IB_PFONT1, 22,124, "Quickly tap the text area to", 152 );
            LibPutProStr( IB_PFONT1, 22,132, "write punctuations and other", 152 );
            LibPutProStr( IB_PFONT1, 22,140, "characters like '&'... or use", 152 );
            LibPutProStr( IB_PFONT1, 22,148, "the middle mode-indicator.", 152 );
            LibBox( 9, 130, 3, 3, 1);
            if (!waitfornextpage())
                break;

            LibPutProStr( IB_PFONT2, 5,  5, "Numeric-Mode:", 152 );
            LibPutProStr( IB_PFONT1, 28, 16, "Draw a simple up-right-", 152 );
            LibPutProStr( IB_PFONT1, 28, 24, "stroke to switch to", 152 );
            LibPutProStr( IB_PFONT1, 28, 32, "numeric mode.", 152 );
            LibPutProStr( IB_PFONT1, 28, 40, "Draw the stroke a second", 152 );
            LibPutProStr( IB_PFONT1, 28, 48, "time to switch back to", 152 );
            LibPutProStr( IB_PFONT1, 28, 56, "letter-mode.", 152 );
            drawOnePattern( 2, 20, 0, 38 ); /* NUM_STROKE */
            LibPutProStr( IB_PFONT1, 5, 67, "Alternatively, you can use the", 152 );
            LibPutProStr( IB_PFONT1, 5, 75, "lower mode-indicator. It shows", 152 );
            LibPutProStr( IB_PFONT1, 5, 83, "a bold '1' for numeric mode", 152 );
            LibPutProStr( IB_PFONT1, 5, 91, "and after a second click a '+'", 152 );
            LibPutProStr( IB_PFONT1, 5, 99, "for extended numeric symbols.", 152 );

            LibPutProStr( IB_PFONT2, 5,118, "Extended Numeric Symbols:", 152 );
            LibPutProStr( IB_PFONT1, 22,129, "Quickly tap the text area", 152 );
            LibPutProStr( IB_PFONT1, 22,137, "(in numeric mode) to write", 152 );
            LibPutProStr( IB_PFONT1, 22,145, "characters like '+', '{', ...", 152 );
            LibBox( 9, 130, 3, 3, 1);

            waitfornextpage();
            break;

        case HELPMODE_ALPHA:
            LibPutProStr( IB_PFONT2, 5,  5, "Letters:", 152 );
            drawPatterns( 2, 12, 0, 3, 22 );
            if (!waitfornextpage())
                break;

            drawPatterns( 2,  7, 0, 23, 36 );
            LibPutProStr( IB_PFONT1, 5,116, "In letter and numeric mode you", 152 );
            LibPutProStr( IB_PFONT1, 5,124, "can write:", 152 );
            drawPatterns( 42,120, 0, 0, 2 );
            if (!waitfornextpage())
                break;

            LibPutProStr( IB_PFONT2, 5,  5, "Capital letters:", 152 );
            LibPutProStr( IB_PFONT1, 5, 16, "in capital-mode you can use", 152 );
            LibPutProStr( IB_PFONT1, 5, 24, "all lower-case strokes plus", 152 );
            LibPutProStr( IB_PFONT1, 5, 32, "the following:", 152 );
            drawPatterns( 2, 41, 1, 0, 4 );
            LibPutProStr( IB_PFONT2, 5, 80, "Clipboard:", 152 );
            LibPutProStr( IB_PFONT1, 5, 88, "In letter- and numeric-mode", 152 );
            LibPutProStr( IB_PFONT1, 5, 96, "you can copy to and paste from", 152 );
            LibPutProStr( IB_PFONT1, 5,104, "the PV clipboard.", 152 );
            drawPatterns( 2,116, 0, 44, 45 );
            waitfornextpage();
            break;

        case HELPMODE_NUM:
            LibPutProStr( IB_PFONT2, 5,  5, "Numeric Mode:", 152 );
            LibPutProStr( IB_PFONT1, 28, 16, "Draw a simple up-right-", 152 );
            LibPutProStr( IB_PFONT1, 28, 24, "stroke to switch to", 152 );
            LibPutProStr( IB_PFONT1, 28, 32, "numeric mode.", 152 );
            drawOnePattern( 2, 16, 0, 38 ); /* NUM_STROKE */
            drawPatterns( 2, 46, 3, 3, 15 );
            waitfornextpage();
            break;

        case HELPMODE_EXT:
            LibPutProStr( IB_PFONT2, 5,  5, "Extended Characters:", 152 );
            LibPutProStr( IB_PFONT1, 22, 16, "Quickly tap the text area to", 152 );
            LibPutProStr( IB_PFONT1, 22, 24, "switch to extended-mode.", 152 );
            LibPutProStr( IB_PFONT1, 5, 32, "Tap a second time to write '.'.", 152 );
            LibBox( 9, 20, 3, 3, 1);
            drawPatterns( 2, 46, 2, 1, 15);
            if (!waitfornextpage())
                break;

            drawPatterns( 2, 12, 2, 16, 22);
            waitfornextpage();
            break;

        case HELPMODE_EXT_NUM:
            LibPutProStr( IB_PFONT2, 5,  5, "Extended Numeric Symbols:", 152 );
            LibPutProStr( IB_PFONT1, 22, 16, "Quickly tap the text area in", 152 );
            LibPutProStr( IB_PFONT1, 22, 24, "numeric-mode to switch to", 152 );
            LibPutProStr( IB_PFONT1, 22, 32, "extended-numeric-mode.", 152 );
            LibBox( 9, 28, 3, 3, 1);
            drawPatterns( 2, 46, 4, 1, 12);
            waitfornextpage();
            break;

        case HELPMODE_ACC:
            LibPutProStr( IB_PFONT2, 5,  5, "Accents and Umlauts:", 152 );
            LibPutProStr( IB_PFONT1, 5, 16, "First write the letter that", 152 );
            LibPutProStr( IB_PFONT1, 5, 24, "you wish to put the accent on.", 152 );
            LibPutProStr( IB_PFONT1, 5, 32, "Then write one of the following", 152 );
            LibPutProStr( IB_PFONT1, 5, 40, "accent-stroke.", 152 );
            drawPatterns( 2, 52, 0, 39, 43);
            LibPutProStr( IB_PFONT1, 5, 90, "Please note that the first", 152 );
            LibPutProStr( IB_PFONT1, 5, 98, "accent is a straight down-and-", 152 );
            LibPutProStr( IB_PFONT1, 5,106, "left line that goes back up to", 152 );
            LibPutProStr( IB_PFONT1, 5,114, "the starting point.", 152 );
            LibPutProStr( IB_PFONT1, 5,122, "The 2nd and 3rd pattern start", 152 );
            LibPutProStr( IB_PFONT1, 5,130, "like the 1st. The 2nd then goes", 153 );
            LibPutProStr( IB_PFONT1, 5,138, "down-left a second time, while", 152 );
            LibPutProStr( IB_PFONT1, 5,146, "the 3rd goes down-right instead.", 152 );
            waitfornextpage();
            break;

        case HELPMODE_ABOUT:
            LibPutProStr( IB_PFONT2, 5,  5, "PV-Merlin Version 3-beta", 152 );
            LibPutProStr( IB_PFONT1, 5, 16, "The original pattern-recognition", 152 );
            LibPutProStr( IB_PFONT1, 5, 24, "algorithms and most of the non-", 152 );
            LibPutProStr( IB_PFONT1, 5, 32, "GUI-code comes from and are", 152 );
            LibPutProStr( IB_PFONT1, 5, 40, "copyrighted by Stefan Hellkvist.", 152 );
            LibPutProStr( IB_PFONT1, 5, 50, "Wittawat Yamwong did a very", 152 );
            LibPutProStr( IB_PFONT1, 5, 58, "nice port of Stefan's 'xmerlin'", 152 );
            LibPutProStr( IB_PFONT1, 5, 66, "to the PocketViewer.", 152 );
            LibPutProStr( IB_PFONT1, 5, 76, "I, Christian Ensel, improved the", 152 );
            LibPutProStr( IB_PFONT1, 5, 84, "character recogintion and wrote", 152 );
            LibPutProStr( IB_PFONT1, 5, 92, "the pen handling and GUI.", 152 );
            LibPutProStr( IB_PFONT1, 5,102, "Marcin Wolinski helped with the", 152 );
            LibPutProStr( IB_PFONT1, 5,110, "GUI and wrote the file access.", 152 );
            LibPutProStr( IB_PFONT1,25,138, "Thanks to all, have fun!", 152 );
            LibPutProStr( IB_PFONT1,140,146, "ce", 152 );
            waitfornextpage();
            break;
    }

    LibCloseWindow();    
    LibPutDisp();
}














