/****************************************************************************
 * pattern.c
 *
 * Copyright (c) 2000 Stefan Hellkvist, Ericsson Radio Systems AB
 * Email: stefan@hellkvist.org
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *****************************************************************************/

/* Oct 2000 - ported to Pocket Viewer by Wittawat Yamwong <wittawat@web.de> */

#include <stdrom.h>

#include "md.h"
#include "mm.h"

#include "pattern.h"

Pattern FAR *
newPattern( int initialSize, unsigned char ascii )
{
    Pattern FAR *p = (Pattern FAR *) mm_malloc ( sizeof( Pattern ) );
    if ( p == NULL )
        return NULL;
    p->pv = (Point FAR *) mm_malloc ( initialSize * sizeof( Point ) );
    if ( p->pv == NULL )
    {
        free( p );
        return NULL;
    }

    p->s = NULL;
    p->nPoints = 0;
    p->ascii = ascii;
    p->size = initialSize;

    return p;
}


void
deletePattern( Pattern FAR *p )
{
    if ( p->s != NULL)
        free( p->s );
    free( p->pv );
    free( p );
}

static
Point FAR *
growVector( Point FAR *old, int oldSize, int newSize )
{
    Point FAR *np = (Point FAR *) mm_malloc( newSize * sizeof( Point ) );
    if (np == NULL)
        return NULL;
    far_memcpy( np, old, oldSize * sizeof( Point ) );
    return np;
}

#define MIN_RAWPATTERN_POINT_DIST 2

void
addPoint( Pattern FAR *p, Point po )
{
    /* po.isLast = 0; */
    /* do not add (almost) identical points */
    if ( p->nPoints>0 ) { 
        register d = p->pv[p->nPoints-1].x - po.x; 
        if ( d < MIN_RAWPATTERN_POINT_DIST &&
             d > -MIN_RAWPATTERN_POINT_DIST ) {
                d = p->pv[p->nPoints-1].y - po.y;
                if ( d < MIN_RAWPATTERN_POINT_DIST &&
                     d > -MIN_RAWPATTERN_POINT_DIST ) {
                    return;
                }
        }
    }
    p->pv[ p->nPoints++ ] = po;
    if ( p->nPoints == p->size )
    {
        Point far *old = p->pv;
        int newSize = p->size * 2;
        p->pv = growVector( p->pv, p->size, newSize );
        if ( p->pv != NULL)  /* Did we manage to grow? */
        {
            p->size = newSize;  /* Yes, use new vector */
            free( old );
        }
        else
        {
            p->pv = old;  /* No, use old */
            p->nPoints--; /* Go back one step */
        }
    }
}

#include "patternf.c"