/*****************************************************************************
 * preprocessor.c
 *
 * Copyright (c) 2000 Stefan Hellkvist, Ericsson Radio Systems AB
 * Email: stefan@hellkvist.org
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *****************************************************************************/

/* Oct 2000 - ported to Pocket Viewer by Wittawat Yamwong <wittawat@web.de> */

#include <stdrom.h>

#include "md.h"
#include "mm.h"

#include "preproc.h"
#include "pattern.h"

Preprocessor FAR *
newPreprocessor( filter *fs, int nFilters )
{
    Preprocessor FAR *pnew = (Preprocessor FAR *) mm_malloc( sizeof( Preprocessor ) );
    if (pnew == NULL)
	return NULL;
    pnew->fs = fs;
    pnew->nFilters = nFilters;
    return pnew;
}

void
deletePreprocessor( Preprocessor FAR *p )
{
    free( p );
}

static
Pattern FAR *
applyFilter( filter f, Pattern FAR *p)
{
    return f( p );
}

/*ce: Comment: a new pattern is allocated in memory, the old 
      one is preserved. (intermediate patterns are deleted)  
*/

Pattern FAR *
apply( Preprocessor FAR *p, Pattern FAR *pat )
{
    Pattern FAR *np, FAR *old;
    int i;
    if ( p->nFilters == 0 )
	return pat;

    np = applyFilter( p->fs[ 0 ], pat );
    for ( i = 1; i < p->nFilters; i++)
    {
	old = np;
	np = applyFilter( p->fs[ i ], np );
	deletePattern( old );
    }
    return np;
}
