#include <stdrom.h>

#include "define.h"
#include "libc.h"
#include "l_define.h"
#include "l_libc.h"

#include "md.h"

#include "mm.h"
#include "event.h"
#include "mod_symb.h"

#include "modebut.h"

#include "ml_help.h"

#include "permanent.h"



#define HEAP_SIZE  (20*1024) 
#define OBJ_MERLIN 0x8001
#define OBJ_MYHARDICON 0x8002

/*** Designation of text area ***/
#define M_ST_X 2 /* st_x */
#define M_ST_Y 15 /* st_y */
#define M_ED_X (M_ST_X+ML_WIDTH+2) /* ed_x */
#define M_IT_Y 9 /* it_y */
#define M_MAXG ((ML_HEIGHT+2)/(M_IT_Y)) /* MAXGYO */
/*** Scroll bar setting ***/
#define SCR__Y M_ST_Y /* Scroll bar position */
#define SCR__SIZE M_IT_Y*M_MAXG /* Height of scroll bar */
#define SCR__DSP M_MAXG /* Number of display records */

#define PVM_PERMANENT_FILENAME "pvmerlin.dat"

/*data stored permanently in pvm-file */
static struct s_pvmerlin_permanent_data {
    byte mtxbf[2049]; /* Buffer for text input */
    int cursorpos;
};

struct s_pvmerlin_permanent_data* p_pvmerlin_permanent_data;

static TCHTBL InptTch[3]; /* Text touch area table */
static u8 far heap[HEAP_SIZE];
static int x,y;
static byte curch;

static TCHTBL far JotPad[] = {
       { MERLIN_TCH(M_ST_X+1, M_ST_Y+1, OBJ_MERLIN) },
/*       { 140,164, 164,177, ACT_MAKE, OBJ_HIC_ESC }, */
/*      {  17,164, 139,200, ACT_MAKE, OBJ_MYHARDICON },  */
       { 0,0,0,0,0,OBJ_END,0 }
};

/*ce original line: { MERLIN_TCH(40, 160-ML_HEIGHT, OBJ_MERLIN) }, */

/*ce conversion from simple to accented characters and umlauts */

char accent[] = { 0x11, '"', 0x13, 0x12, 0x14, 0 }; /* ,",^,`,~ */
char simplechars[]     = "aeiouAEIOUnNyYcC"; /* used to find the position within the accented strings */
char accentedchars[5][16]= { {0x90, 0x91, 0x92, 0x93, 0x94, 0x80, 0x81, 0x82, 0x83, 0x84, 0, 0, 0xe3, 0xe4, 0xba, 0xaa },
                           {0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0, 0, 0xd8, 0, 0, 0 },
                           {0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0, 0, 0, 0, 0, 0},
                           {0x95, 0x96, 0x97, 0x98, 0x99, 0x85, 0x86, 0x87, 0x88, 0x89, 0, 0, 0, 0, 0xba, 0xaa },
                           {0xb5, 0, 0, 0xb6, 0, 0xa5, 0, 0, 0xa6, 0, 0xb7, 0xa7, 0, 0, 0, 0}
                         }; /* the maching accented characters */


static void drawBackground()
{
	LibClrDisp();
	mlPaint();
        LibBox(M_ST_X-2, M_ST_Y-2, M_ED_X+SCR_XSIZE+6, M_MAXG*M_IT_Y+3, 1);
}

static void initApp()
{
	mm_create_heap(heap, HEAP_SIZE);
    mlInit(JotPad, TRUE);
}

static void showChar()
{
	byte buf[3];

	if (curch >= 0x20) {
		buf[0] = curch;
		buf[1] = '\0';
	} else {
		buf[0] = '^';
		buf[1] = curch + 0x40;
		buf[2] = 0;
	}
	LibClrBox(0,0,20,20);
	LibPutProStr(IB_PFONT3, 0, 0, buf, 20);
	LibPutDisp();
}



/*********** titlebuttons (ce) **************/

#define NRTITLEBUTTONS 8
#define TITLEBUTTON_MODECHANGE 0
#define TITLEBUTTON_TIME   1
#define TITLEBUTTON_CALC   2
#define TITLEBUTTON_DEL    3
#define TITLEBUTTON_HELP   4
#define TITLEBUTTON_TOOLS  5
#define TITLEBUTTON_PGUP   6
#define TITLEBUTTON_PGDN   7
#define TITLEBUTTON_OBJ_BASE 0x8080

static TCHTBL far titleButTchTbl[] = {
       {   0, 0,   9, 11, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_MODECHANGE, 0x0000 },
       {  85, 0,  96, 11, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_TIME, 0x0000 },
       {  99, 0, 107, 11, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_CALC, 0x0000 },
       { 111, 0, 129, 11, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_DEL, 0x0000 },
       { 131, 0, 139, 11, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_HELP, 0x0000 },
       { 140, 0, 158, 11, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_TOOLS, 0x0000 },
       { 158-MODEBUTTON_XSIZE, 13, 159, 23, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_PGUP, 0x0000 },
       { 158-MODEBUTTON_XSIZE, 24, 159, 36, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_PGDN, 0x0000 },
       { 0,0,0,0,ACT_NONE,OBJ_END,0x0000 }
};

static T_ICON far titleButIcons[] = {
     {&titleButTchTbl[0],NULL,NULL,0x03},
     {&titleButTchTbl[1],NULL,NULL,0x03},
     {&titleButTchTbl[2],NULL,NULL,0x01},
     {&titleButTchTbl[3],NULL,NULL,0x02},
     {&titleButTchTbl[4],NULL,NULL,0x03}, /* 01 */
     {&titleButTchTbl[5],NULL,NULL,0x02},
     {&titleButTchTbl[6],NULL,NULL,0x03},
     {&titleButTchTbl[7],NULL,NULL,0x03}
};

void titleButtonsInit()
{
    int x,y;
    
    /* help+tools+calc+time+del-buttons */
    LibPutFarData(160-30, titleButTchTbl[TITLEBUTTON_TOOLS].y1, 131);
    LibPutFarData(titleButTchTbl[TITLEBUTTON_TIME].x1-2, titleButTchTbl[TITLEBUTTON_TIME].y1, 133);
    x = titleButTchTbl[TITLEBUTTON_TOOLS].x1;
    y = titleButTchTbl[TITLEBUTTON_TOOLS].y1;
    LibLineClr( x, y, 1, 12 );
    LibDotOff( x+1, y+11 );

    x = titleButTchTbl[TITLEBUTTON_HELP].x1;
    y = titleButTchTbl[TITLEBUTTON_HELP].y1;
    LibBox( x-2, y+1, 2, 9, 1 );
    LibDotOff( x-1, y );
    LibLineClr( x, y, 1, 12 );
    LibDotOff( x+1, y+11 );

    /* PgUp/PgDn */
    LibPutBoxSub( titleButTchTbl[TITLEBUTTON_PGUP].x1, titleButTchTbl[TITLEBUTTON_PGUP].y1,
                  titleButTchTbl[TITLEBUTTON_PGDN].x2-1, titleButTchTbl[TITLEBUTTON_PGDN].y2-1, IB_GDS_OVER );
    LibGdsLine( titleButTchTbl[TITLEBUTTON_PGDN].x2, titleButTchTbl[TITLEBUTTON_PGUP].y1+1,
                  titleButTchTbl[TITLEBUTTON_PGDN].x2, titleButTchTbl[TITLEBUTTON_PGDN].y2 );
    LibGdsLine( titleButTchTbl[TITLEBUTTON_PGUP].x1+1, titleButTchTbl[TITLEBUTTON_PGDN].y2,
                  titleButTchTbl[TITLEBUTTON_PGDN].x2, titleButTchTbl[TITLEBUTTON_PGDN].y2 );
    LibPutFarData( titleButTchTbl[TITLEBUTTON_PGUP].x1+2, titleButTchTbl[TITLEBUTTON_PGUP].y1+2, 257 );
    LibPutFarData( titleButTchTbl[TITLEBUTTON_PGDN].x1+2, titleButTchTbl[TITLEBUTTON_PGDN].y1+2, 184 );
    LibGdsLineMesh( titleButTchTbl[TITLEBUTTON_PGDN].x1+1, titleButTchTbl[TITLEBUTTON_PGDN].y1,
                    titleButTchTbl[TITLEBUTTON_PGDN].x2-1, titleButTchTbl[TITLEBUTTON_PGDN].y1 );
        
    /* help-button-text */
    LibPutProFont(IB_PFONT1, '?', x+1, y+2 );

    /* del-button-text */
    LibPutProStr(IB_PFONT1, titleButTchTbl[TITLEBUTTON_DEL].x1+3, titleButTchTbl[TITLEBUTTON_DEL].y1+2, "Del", 20 );
    
    /* Tools-button-text */
    x = titleButTchTbl[TITLEBUTTON_TOOLS].x1+2;
    y = titleButTchTbl[TITLEBUTTON_TOOLS].y1+2;
    LibPutProFont(IB_PFONT1, 'T', x, y );
    x += LibGetProFontSize(IB_PFONT1, 'T' )-1;
    LibPutProFont(IB_PFONT1, 'o', x, y );
    x += LibGetProFontSize(IB_PFONT1, 'o' )-1;
    LibPutProFont(IB_PFONT1, 'o', x, y );
    x += LibGetProFontSize(IB_PFONT1, 'o' )-1;
    LibPutProFont(IB_PFONT1, 'l', x, y );

    
    /* update display */
    x = titleButTchTbl[0].x1;
    y = titleButTchTbl[0].y1;
    LibPutDispBox( x, y, titleButTchTbl[NRTITLEBUTTONS-1].x1 - x, titleButTchTbl[NRTITLEBUTTONS-1].y1 - y);
    
    /* setup touch table */
  	LibTchStackPush( titleButTchTbl );
}


void titleButtonsPenEvents(TCHSTS *p_tsts, TXTP *p_mintp)
{
    int buttonnr = p_tsts->obj - TITLEBUTTON_OBJ_BASE;
    if ( buttonnr < 0 || buttonnr >= NRTITLEBUTTONS )
        return;

    if ( LibIconClick( &titleButIcons[buttonnr], p_tsts ) == TRUE )  {
        /* a button was pressed */
        switch ( buttonnr ) {
            case TITLEBUTTON_MODECHANGE:
                break;
            case TITLEBUTTON_HELP:
                merlinhelpmenu();
                break;
            case TITLEBUTTON_TOOLS:
                LibFuncPtool();
                break;
            case TITLEBUTTON_CALC:
                LibCalWin(ON);
                break;
            case TITLEBUTTON_TIME:
                p_tsts->obj = OBJ_TXTTCH;
                p_mintp->txtst = TXTDYTIM;
                break;
            case TITLEBUTTON_DEL:
                p_tsts->obj = OBJ_TXTTCH;
                p_mintp->txtst = TXTDEL;  /* does not work; see further down for work around */
                break;
            case TITLEBUTTON_PGUP:
                LibPutMsgDlg("PgUp not implemented,yet");
                break;
            case TITLEBUTTON_PGDN:
                LibPutMsgDlg("PgDn not implemented,yet");
                break;
        }
        
    }

}

void translate(int *ch)
{
    if (*ch<8)  /*ce - those values are used for the special copy/past/...-strokes */
        *ch = KEY_NONE;
        
    switch (*ch) {
    case 0x0D:
        *ch = KEY_RTN;
        break;

    case 0x08:
        *ch = KEY_DEL;
        break;

    case ' ':
        *ch = KEY_SPC;
        break;
    }
}

/********** main event handling ****************/

void backspace(TCHSTS *p_tsts, TXTP *p_mintp)
{
    if ( p_mintp->cp > 0) {
        p_mintp->sr_sp = p_mintp->cp-1;
        p_mintp->sr_ep = p_mintp->cp;
        p_mintp->selrv = TRUE;
        
        LibTxtInp(KEY_NONE,p_tsts,p_mintp);
        if(LibTxtDspC(p_mintp)==TRUE)
            LibPutDisp();
            
        LibTxtInp(KEY_DEL,p_tsts,p_mintp);
        if(LibTxtDspC(p_mintp)==TRUE)
            LibPutDisp();
    }                 
}

typedef enum { PVM_IDLE_MODE, PVM_PENSTART_MODE, PVM_WRITE_MODE, PVM_TXTCURSOR_MODE } t_pvm_mode;

#define TXTCURSOR_TIME_THRESHOLD 4
#define TXTCURSOR_DIST_THRESHOLD 2

void eventLoop(void)
{
    int ch;
    int thischarmode;
    bool con = FALSE;
    TCHSTS tsts;
    TXTP m_in_tp; /* Declaration of structure for text input*/
    T_SCR_POS m_in_scb; /* Scroll bar position information */
  	
  	s_modeButtonsPos mbPos = { { 159-MODEBUTTON_XSIZE, 159-MODEBUTTON_XSIZE, 159-MODEBUTTON_XSIZE },
  	                           { 38,  38+MODEBUTTON_YSIZE,  38+2*MODEBUTTON_YSIZE }  };    /*ce Merlin-ModeButtons */
    
    t_pvm_mode pvmmode = PVM_IDLE_MODE;
    int pendown_count = 0;
    int pendown_x, pendown_y;
        
    /* init permanent data */
    if ( !permanentInit(PVM_PERMANENT_FILENAME, &p_pvmerlin_permanent_data, sizeof(struct s_pvmerlin_permanent_data) ) ) {
        /* no file -> manual initalization */
        /*p_pvmerlin_permanent_data->mtxbf[0] = 0; /* Initialization of text buffer */
    }
    
    /*** Text input setting ***/
    m_in_tp.st_x = M_ST_X; /* Start coordinate (X) of text display */
    m_in_tp.st_y = M_ST_Y; /* Start coordinate (Y) of text display */
    m_in_tp.ed_x = M_ED_X; /* End coordinate (X) of text display */
    m_in_tp.it_y = M_IT_Y; /* Text display line spacing (Y) */
    m_in_tp.MAXGYO = M_MAXG; /* Number of text display lines */
    m_in_tp.font = IB_PFONT1; /* Display font type */
    m_in_tp.csen = TRUE; /* Cursor display enabled */
    m_in_tp.rtnen = TRUE; /* CR code display enabled */
    m_in_tp.maxmj = 2048; /* Maximum number of allowable input characters. */
    m_in_tp.txbf = p_pvmerlin_permanent_data->mtxbf; /* Designation of text buffer address */
    m_in_tp.gdcmt = NULL; /* Guidance comment table */
    m_in_tp.txtobj = OBJ_TXTTCH; /* Object when text area is touched. */
    m_in_tp.sbrobj = OBJ_SCR_BAR; /* Object when the scroll bar is touched. */
    m_in_tp.tchtb = InptTch; /* Pointer for text scroll bar area */
    m_in_tp.s_pos = &m_in_scb; /* Pointer for text and scroll bar information */
    /*for (i=0; i<20; i++)
        mtxbf[i]=0xD;
    mtxbf[20] = 0;*/
    

    LibTxtInit(&m_in_tp); /* Initialization of text input */
    m_in_scb.x += 1;
    m_in_scb.y += 60;
    m_in_scb.size -= 60;
    m_in_tp.cp = p_pvmerlin_permanent_data->cursorpos;

    /*** Text touch area setting ***/
    LibTxtTchSet(&m_in_tp); /* Text touch area PUSH */
    
    /*LibTchStackPop();*/
  	LibTchStackPush(JotPad);

    /*ce some cosmetics for the modeButtons and pgup/pgdn */  	
    LibGdsLineClr( mbPos.x[0], 13, 159, 13 );
    /* LibGdsLine( mbPos.x[0], mbPos.y[0]-1, 159, mbPos.y[0]-1 ); */
    LibGdsLine( mbPos.x[0]-1, 13, mbPos.x[0]-1 , mbPos.y[NRMODEBUTTONS-1] + MODEBUTTON_YSIZE -1 ); 
    /*LibGdsLineMesh( mbPos.x[0]-1, mbPos.y[NRMODEBUTTONS-1]+MODEBUTTON_YSIZE, 159, mbPos.y[NRMODEBUTTONS-1] + MODEBUTTON_YSIZE ); */
    LibGdsLineClr( 159, mbPos.y[0]-1, 159 , 159 ); /* mbPos.y[NRMODEBUTTONS-1] + MODEBUTTON_YSIZE -1 ); */

  	/*ce Merlin-ModeButtons and titleButtons */  	
  	modeButtonsInit( &mbPos );
    titleButtonsInit();
      	
	LibTchInit();

    
    /*** Data input loop ***/
    while (1){
        if(LibTxtDspC(&m_in_tp)==TRUE) /* Display during text input */
            LibPutDisp();
        if (m_in_tp.sr_sp == m_in_tp.sr_ep || !m_in_tp.selrv) {
            LibCurBlnkOn(m_in_tp.c_xp, m_in_tp.c_yp, 2, m_in_tp.it_y);
            con = TRUE;
        } else {
            LibCurBlnkOff();
            con = FALSE;
        }
        LibTchWait(&tsts);
        /*LibPutMsgDlg3("ev %x %x", tsts.act, tsts.obj );*/
        
        /*if (tsts.obj == OBJ_MERLIN )
            ch = mlHandleEvent(&tsts);
        else */

        thischarmode = getCharMode();
        ch = -1;
        
        /* mode buttons */
        if (tsts.obj >= MODEBUTTON_OBJ_BASE  &&  tsts.obj < MODEBUTTON_OBJ_BASE+NRMODEBUTTONS ) {
            modeButtonsPenEvent(&tsts);
        }
        
        /* title buttons */
        else if (tsts.obj >= TITLEBUTTON_OBJ_BASE  &&  tsts.obj < TITLEBUTTON_OBJ_BASE+NRTITLEBUTTONS ) {
            LibCurBlnkOff();
            titleButtonsPenEvents( &tsts, &m_in_tp );
            if (m_in_tp.txtst == TXTDEL) {
                ch = KEY_DEL;
                m_in_tp.txtst = TXTNOP;
            }
            LibCurBlnkOn(m_in_tp.c_xp, m_in_tp.c_yp, 2, m_in_tp.it_y);
        }
        
        /* esc */
        else if (tsts.obj == OBJ_HIC_ESC ) {
            p_pvmerlin_permanent_data->cursorpos = m_in_tp.cp;
            permanentSave();
            LibJumpMenu();
        }

        /* other hard icons -- does not work *
        else if ( tsts.obj == OBJ_MYHARDICON ) {
               *(tsts.obj >= OBJ_HIC_POS1 && tsts.obj <= OBJ_HIC_POS5 )  { *
            p_pvmerlin_permanent_data->cursorpos = m_in_tp.cp;
            permanentSave();
        	LibTchStackPush(TchHardIcon);
            LibTchHardIcon(&tsts,0);
        }
        */
        
        /* text-field */
        else if (tsts.obj == OBJ_MERLIN || tsts.obj == OBJ_TXTTCH ) {  /*ce all new */
            switch (pvmmode) {
                register dx,dy;
                case PVM_IDLE_MODE:
                    if ( tsts.act != ACT_MAKE )
                        break;
                    pvmmode = PVM_PENSTART_MODE;
                    pendown_count = 0;
                    pendown_x = tsts.x;
                    pendown_y = tsts.y;
                case PVM_PENSTART_MODE:
                    pendown_count++;
                    dx = pendown_x - tsts.x;
                    dy = pendown_y - tsts.y;
                    if ( dx > TXTCURSOR_DIST_THRESHOLD || dx < -TXTCURSOR_DIST_THRESHOLD  
                         || dy > TXTCURSOR_DIST_THRESHOLD || dy < -TXTCURSOR_DIST_THRESHOLD) /* pen moved to early -> user is writing a letter */
                        pvmmode = PVM_WRITE_MODE;                        
                    else if (pendown_count > TXTCURSOR_TIME_THRESHOLD) { /* pen did not move for some time -> user placing the cursor */
                        dropInputPattern();
                        LibGdsReverse( mbPos.x[0], mbPos.y[0], mbPos.x[NRMODEBUTTONS-1]+MODEBUTTON_XSIZE-1, mbPos.y[NRMODEBUTTONS-1]+MODEBUTTON_YSIZE-1 ); /* turn indicator on */
                        LibPutDispBox( mbPos.x[0], mbPos.y[0], mbPos.x[NRMODEBUTTONS-1]+MODEBUTTON_XSIZE-mbPos.x[0], mbPos.y[NRMODEBUTTONS-1]+MODEBUTTON_YSIZE-mbPos.y[0] );
                        pvmmode = PVM_TXTCURSOR_MODE;
                        tsts.obj = OBJ_TXTTCH;
                        tsts.act = ACT_MAKE;
                        break;
                    }
                case PVM_WRITE_MODE:
                    if ( tsts.act == ACT_MOVE_OUT ) {
                        dropInputPattern();
                        pvmmode = PVM_IDLE_MODE;
                        break;
                    }
                    ch = mlHandleEvent(&tsts);
                    if ( ch != -1 )
                        pvmmode = PVM_IDLE_MODE;
                    break;
                case PVM_TXTCURSOR_MODE:
                    if ( tsts.act == ACT_BREAK_IN /* || tsts.act == ACT_MOVE_OUT */
                         || tsts.act == ACT_BREAK) {
                        LibGdsReverse( mbPos.x[0], mbPos.y[0], mbPos.x[NRMODEBUTTONS-1]+MODEBUTTON_XSIZE-1, mbPos.y[NRMODEBUTTONS-1]+MODEBUTTON_YSIZE-1 ); /* turn indicator on */
                        LibPutDispBox( mbPos.x[0], mbPos.y[0], mbPos.x[NRMODEBUTTONS-1]+MODEBUTTON_XSIZE-mbPos.x[0], mbPos.y[NRMODEBUTTONS-1]+MODEBUTTON_YSIZE-mbPos.y[0] );
                        pvmmode = PVM_IDLE_MODE;
                        /* tsts.act = ACT_BREAK_IN; */
                    }
                    tsts.obj = OBJ_TXTTCH;
                    break;
            }
        }

        if (ch==0x08) { /* BackSpace */
            if (  m_in_tp.selrv != TRUE ) {
                backspace(&tsts,&m_in_tp);
                ch = KEY_NONE;
            }
        } 
        else if ( ch==COPY_STROKE ) { 
            if (  m_in_tp.selrv == TRUE ) {
                m_in_tp.txtst = TXTCOPY;
            } else {
                int cp = m_in_tp.cp;
                /* nothing selected: temporarily select whole text area */
                m_in_tp.sr_sp = 0;
                m_in_tp.sr_ep = strlen(m_in_tp.txbf);  
                m_in_tp.selrv = TRUE;
                LibTxtInp(KEY_NONE,&tsts,&m_in_tp);
                if(LibTxtDspC(&m_in_tp)==TRUE)
                    LibPutDisp();
                /* copy */
                m_in_tp.txtst = TXTCOPY;
                LibTxtInp(KEY_NONE,&tsts,&m_in_tp);
                if(LibTxtDspC(&m_in_tp)==TRUE)
                    LibPutDisp();
                /* delselect */
                m_in_tp.cp = cp;
                m_in_tp.sr_sp = 0;
                m_in_tp.sr_ep = 0;  
                m_in_tp.selrv = FALSE;
                LibTxtInp(' ',&tsts,&m_in_tp);
                /*if(LibTxtDspC(&m_in_tp)==TRUE)
                    LibPutDisp();*/
                backspace(&tsts,&m_in_tp);
            }
        }
        else if ( ch==PASTE_STROKE ) { 
            m_in_tp.txtst = TXTPASTE;
        }
    
        translate(&ch);

        /* test for accents/umlaut-strokes */
        if ( ch!=KEY_NONE && m_in_tp.cp > 0 && thischarmode==ALPHA_MODE ) {
            int prevchar;
            char *pc;
            int i_ac, i_pc;

            prevchar = m_in_tp.txbf[m_in_tp.cp-1];
            pc = strchr( accent, ch );
            if ( pc!=NULL ) { /* new char is an accent  */
                i_ac = pc - accent;
                pc = strchr( simplechars, prevchar );
                if ( pc!=NULL ) { /* prev. character is accentable */
                    i_pc = pc - simplechars;
                    ch = accentedchars[i_ac][i_pc];  /* set new char */
                    if (ch==0)
                        ch = KEY_NONE;
                    else
                        backspace(&tsts,&m_in_tp);   /* delete prev. char */
                }
            }
        }
        
        /* LibPutMsgDlg3("m: %i", pvmmode );*/

        LibTxtInp(ch,&tsts,&m_in_tp);
    }
    if (con)  
        LibCurBlnkOff();
}

#if 0
static void penEvent( const TCHSTS *tsts)
{
	Point p;
	bool draw = TRUE;
	int i;

	p.x = tsts->x; p.y = tsts->y;
	switch (tsts->act) {
	case ACT_MAKE:
		if (current)
			deletePattern(current);
		mlPaint();
		current = newPattern(100,'a');
		addPoint(current, p);
		x = p.x; y = p.y;
		break;

	case ACT_MOVE:
	case ACT_BREAK_IN:
		addPoint(current, p);
		break;

	case ACT_MOVE_OUT:
		addPoint(current, p);
		LibTchInit();
		break;

	default:
		draw = FALSE;
	}
	if (draw) {
		i = current->nPoints - 1;
		LibGdsLine(x,y,current->pv[i].x,current->pv[i].y);
		LibPutDisp();
		x = p.x; y = p.y;
	}
}

static void print(char *str)
{
	int len = strlen(str);
	int i = 0;

	while (i < 3 && LibSrlSendBlock(str, len) != IW_SRL_NOERR) {
		LibWait(IB_125MWAIT);
		i++;
	}

}

static void send()
{
	char buf[20];
	int i;
	SRL_STAT sr;

	sr.port = IB_SRL_COM2;
	sr.speed = IB_SRL_38400BPS;
	sr.parit = IX_SRL_NONE;
	sr.datab = IX_SRL_8DATA;
	sr.stopb = IX_SRL_1STOP;
	sr.fctrl = IX_SRL_RSCS;
	LibSrlPortOpen(&sr);

	sprintf(buf,"S %d %d\r\n",curch,current->nPoints);
	print(buf);
	for (i=0; i < current->nPoints; i++) {
		sprintf(buf,"P %d %d\r\n",
			current->pv[i].x - JotPad[0].x1,
			current->pv[i].y - JotPad[0].y1);
		print(buf);
	}

	LibSrlPortClose();
}
#endif

main()
{
    int x;
    
	initApp();
	drawBackground();
	LibPutDisp();

	LibTchStackClr();
    LibTchStackPush(NULL);
	LibTchStackPush(TchHardIcon);
	/*ce moved
	LibTchStackPush(JotPad);
	LibTchInit();
	*/

    x = 10;
    LibPutProFont(IB_PFONT3, 'P', x, 1 );
    x += LibGetProFontSize(IB_PFONT3, 'P' )-2;
    LibPutProFont(IB_PFONT3, 'V', x, 1 );
    x += LibGetProFontSize(IB_PFONT3, 'V' )-0;
    LibPutProFont(IB_PFONT3, 'M', x, 1 );
    x += LibGetProFontSize(IB_PFONT3, 'M' )-1;
    LibPutProFont(IB_PFONT3, 'e', x, 1 );
    x += LibGetProFontSize(IB_PFONT3, 'e' )-1;
    LibPutProFont(IB_PFONT3, 'r', x, 1 );
    x += LibGetProFontSize(IB_PFONT3, 'r' )-2;
    LibPutProFont(IB_PFONT3, 'l', x, 1 );
    x += LibGetProFontSize(IB_PFONT3, 'l' )-0;
    LibPutProFont(IB_PFONT3, 'i', x, 1 );
    x += LibGetProFontSize(IB_PFONT3, 'i' )-0;
    LibPutProFont(IB_PFONT3, 'n', x, 1 );
    x += LibGetProFontSize(IB_PFONT3, 'n' )+1;
    LibPutProFont(IB_PFONT3, '3', x, 1 );
    x += LibGetProFontSize(IB_PFONT3, '3' )+0;
    LibPutProStr(IB_PFONT1, x, 4, "beta", 159 );

    LibGdsLine( 7, 0, 7, 7 );
    LibGdsLine( 7, 0, 0, 7 );
    LibGdsLine( 7, 2, 2, 7 );
    LibGdsLine( 7, 4, 4, 7 );
    LibGdsLine( 0, 8, 7, 8 );

#if 0
	LibKeyInit();
	LibDispKey();

	while (1) {
		ch = LibGetKeyM(&tsts);
		if (ch == KEY_NONE) {
			if (tsts.obj == OBJ_MERLIN)
				penEvent(&tsts);
			continue;
		}
		switch (ch) {
		case KEY_RTN:
			curch = 13;
			break;

		case KEY_DEL:
			curch = 8;
			break;

		case KEY_NEXT:
			send();
			break;

		default:
			curch = ch;
		}
		showChar();
	}
#else
        eventLoop();
#endif
}
